/*
 * include/linux/dyn_pageflags.h
 *
 * Copyright (C) 2004-2006 Nigel Cunningham <nigel@suspend2.net>
 *
 * This file is released under the GPLv2.
 *
 * It implements support for dynamically allocated bitmaps that are
 * used for temporary or infrequently used pageflags, in lieu of
 * bits in the struct page flags entry.
 */

#ifndef DYN_PAGEFLAGS_H
#define DYN_PAGEFLAGS_H

#include <linux/mm.h>

typedef unsigned long *** dyn_pageflags_t;

#define BITNUMBER(page) (page_to_pfn(page))

#if BITS_PER_LONG == 32
#define UL_SHIFT 5
#else 
#if BITS_PER_LONG == 64
#define UL_SHIFT 6
#else
#error Bits per long not 32 or 64?
#endif
#endif

#define BIT_NUM_MASK (sizeof(unsigned long) * 8 - 1)
#define PAGE_NUM_MASK (~((1 << (PAGE_SHIFT + 3)) - 1))
#define UL_NUM_MASK (~(BIT_NUM_MASK | PAGE_NUM_MASK))

#define BITS_PER_PAGE (PAGE_SIZE << 3)
#define PAGENUMBER(zone_offset) (zone_offset >> (PAGE_SHIFT + 3))
#define PAGEINDEX(zone_offset) ((zone_offset & UL_NUM_MASK) >> UL_SHIFT)
#define PAGEBIT(zone_offset) (zone_offset & BIT_NUM_MASK)

#define PAGE_UL_PTR(bitmap, zone_num, zone_pfn) \
       ((bitmap[zone_num][PAGENUMBER(zone_pfn)])+PAGEINDEX(zone_pfn))

/* With the above macros defined, you can do...

#define PagePageset1(page) (test_dynpageflag(&pageset1_map, page))
#define SetPagePageset1(page) (set_dynpageflag(&pageset1_map, page))
#define ClearPagePageset1(page) (clear_dynpageflag(&pageset1_map, page))
*/

#define BITMAP_FOR_EACH_SET(bitmap, counter) \
	for (counter = get_next_bit_on(bitmap, -1); counter < max_pfn; \
		counter = get_next_bit_on(bitmap, counter))

extern void clear_dyn_pageflags(dyn_pageflags_t pagemap);
extern int allocate_dyn_pageflags(dyn_pageflags_t *pagemap);
extern void free_dyn_pageflags(dyn_pageflags_t *pagemap);
extern int dyn_pageflags_pages_per_bitmap(void);
extern int get_next_bit_on(dyn_pageflags_t bitmap, int counter);
extern unsigned long *dyn_pageflags_ul_ptr(dyn_pageflags_t *bitmap,
		struct page *pg);

extern int test_dynpageflag(dyn_pageflags_t *bitmap, struct page *page);
extern void set_dynpageflag(dyn_pageflags_t *bitmap, struct page *page);
extern void clear_dynpageflag(dyn_pageflags_t *bitmap, struct page *page);
#endif
